#ifndef _HASH_H
#define _HASH_H

#include "common.h"

/* Definitions for abstract hash table. */

// A hash table is type hash_t
typedef struct hash_data_t *hash_t;

// The keys and values can be anything
typedef void *hash_key_t;
typedef void *hash_value_t;

// The client "knows" that a struct hash_tuple_t is
// a two element structure, and that a hash_tuple_t
// is a pointer to such a structure
struct hash_tuple_t {
  hash_key_t   k;
  hash_value_t v;
};
typedef struct hash_tuple_t * hash_tuple_t;

// The client supplies a function to hash the key to an unsigned long int,
// and another function that can compare two keys for equality.
typedef unsigned long int (*hash_pfhash_t)(hash_key_t);             // hash function type
typedef int (*hash_pfcmp_t)(hash_key_t, hash_key_t);  // element comparison function: returns 0 for equal, -1 for less, 1 for greater

#ifndef FALSE
#define FALSE 0
#define TRUE (!FALSE)
#endif // FALSE

/*
 * Creates and returns a new hash table.
 * Returns null on failure, else a valid hash.
 */
extern hash_t hash_create(hash_pfhash_t, hash_pfcmp_t );

/*
 * Set value for a key.  Returns existing [key,value] tuple via out parameter
 * replacedT if one exists, otherwise NULL. 
 * Returns TRUE for success, FALSE for failure.
 */
extern boolean_t hash_add(hash_t h, hash_tuple_t t, hash_tuple_t* replacedT );

/* Looks up a key in the hash.  If the key is found, places the associated value in
 * out parameter value and returns TRUE.
 * Otherwise, returns FALSE.
 */
extern boolean_t hash_lookup(hash_t h, hash_key_t k, hash_value_t* value);

#endif //_HASH_H


