#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hash.h"

// an array of strings, to be dynamically allocated
static char** string;

static unsigned long int hashFn(hash_key_t k) {
  unsigned int hashVal = 0;
  unsigned int coefficient = 1;
  char* p;
  for ( p = (char*)k; *p; p++  ) {
    hashVal += coefficient * *p;
    coefficient *= 37;
  }
  return hashVal;
}

static int cmpFn(hash_key_t k1, hash_key_t k2) {
  return strcmp((char*)k1, (char*)k2);
}

int main(int argc, char* argv[])
{
  long int i;
  hash_t h = hash_create(hashFn, cmpFn);

  // check for correct invocation
  if ( argc != 2 ) {
    printf("Usage: %s n\nRun test inserting a total of n items\n", argv[0]);
    exit(1);
  }
  int N = atoi(argv[1]);
  string = (char**)malloc(sizeof(char*)*(N+2));

  char stringBuf[32];
  for (i=0; i<N; i++) {
    sprintf( stringBuf, "String %ld", i);
    string[i] = strdup(stringBuf);
  }
  string[i] = strdup(string[i-1]);
  string[i+1] = NULL;

  // First phase -- insert some data
  printf("\nInsert phase:\n");
  for ( i=0; string[i]; i++ ) {
    hash_tuple_t replaced;
    hash_tuple_t t = (hash_tuple_t)malloc(sizeof(struct hash_tuple_t));
    t->k = string[i];
    t->v = (void*)i;
    if ( ! hash_add(h, t, &replaced ) )
      printf("Insertion of %s failed\n", (char*)t->k);
    else if ( replaced )
      printf("Replaced (%s, %ld) while inserting (%s, %ld)\n",
	     (char*)replaced->k, (long int)replaced->v,
	     (char*)t->k, (long int)t->v );
    else
      printf("Inserted %s\n", (char*)t->k);
  }

  // Second phase -- look up some data
  printf("\nLookup phase:\n");
  long int v;
  for ( i=i-1; i>=0; i-- ) {
    if ( ! hash_lookup(h, string[i], (hash_value_t*)&v) ) printf("Lookup of %s failed\n", string[i]);
    else printf("%s -> %ld\n", string[i], v);
  }

  char* notFoundString = "not found string";
  if ( !hash_lookup(h, notFoundString, (hash_value_t*)&v ) ) printf("Lookup of %s failed\n", notFoundString );
  else printf("%s -> %ld\n", notFoundString, v);

  return 0;
}
